VERSION = "2.0.13"
# Echonetlite message format:
#
# - EDT0     |Property value data             |01 ..|01 01 30 01
#  - NUM     |Total number of instances       |01   |1
#  - EOJ     |ECHONET Lite object specificat..|01 ..|01 30 01
#    - EOJX1 |Class group code                |01   |Air conditioner-related device class group
#    - EOJX2 |Class code                      |30   |Home air conditioner class
#    - EOJX3 |Instance code                   |01   |1

ENL_PORT = 3610
ENL_MULTICAST_ADDRESS = "224.0.23.0"

# ------------------------------------------------------------------
# EHD1: ECHONET Lite Header 1
# ---------------------------------------------------------------- */
EHD1 = {
    0x00: 'Not available',
    0x10: 'Conventional ECHONET Lite Specification'
}

# ------------------------------------------------------------------
# EHD1: ECHONET Lite Header 2
# ---------------------------------------------------------------- */
EHD2 = {
    0x81: 'Format 1 (specified message format)',
    0x82: 'Format 2 (arbitrary message format)'
}

# ------------------------------------------------------------------
# ESV
# ------------------------------------------------------------------
GETC = 			0x60
SETC = 			0x61
GET  = 			0x62
INFREQ =		0x63
SETGET = 		0x6E
SETRES =		0x71
GETRES =		0x72
INF =			0x73
INFC = 			0x74
INFC_RES =		0x7A
SETGET_RES =	0x7E
SETI_SNA = 		0x50
SETC_SND =		0x51
GET_SNA = 		0x52
INF_SNA = 		0x53
SETGET_SNA =	0x5E
INSTANCE_LIST = 0xD6

ESV_CODES = {
	0x60: {'name': 'GetC', 'description': 'Property value write request (no response required)'},
	0x61: {'name': 'SetC', 'description': 'Property value write request (response required)'},
	0x62: {'name': 'Get', 'description': 'Property value read request'},
	0x63: {'name': 'INF_REQ', 'description': 'Property value notification request'},
	0x6E: {'name': 'SetGet', 'description': 'Property value write & read request'},
	0x71: {'name': 'Set_Res', 'description': 'Property value Property value write response'},
	0x72: {'name': 'Get_Res' , 'description': 'Property value read response'},
	0x73: {'name': 'INF' , 'description': 'Property value notification'},
	0x74: {'name': 'INFC', 'description': 'Property value notification (response required)'},
	0x7A: {'name': 'INFC_Res' , 'description': 'Property value notification response'},
	0x7E: {'name': 'SetGet_Res' , 'description': 'Property value write & read response'},
	0x50: {'name': 'SetI_SNA', 'description': 'Property value write request (response not possible)'},
	0x51: {'name': 'SetC_SNA' , 'description': 'Property value write request (response not possible)'},
	0x52: {'name': 'Get_SNA', 'description': 'Property value read (response not possible)'},
	0x53: {'name': 'INF_SNA', 'description': 'Property value notification (response not possible)'},
    0x5E: {'name': 'SetGet_SNA', 'description': 'Property value write & read (response not possible)'}
}

MANUFACTURERS = {
    0x000001: 'Hitachi',
    0x000005: 'Sharp',
    0x000006: 'Mitsubishi Electric',
    0x000008: 'DAIKIN INDUSTRIES',
    0x000009: 'NEC',
    0x00000B: 'Panasonic',
    0x00000D: 'Panasonic System Networks',
    0x000012: 'Oi',
    0x000015: 'Daikin Systems & Solutions Laboratory',
    0x000016: 'Toshiba',
    0x000017: 'Toshiba Carrier',
    0x00001B: 'Toshiba Lighting & Technology',
    0x000022: 'Hitach Appliance',
    0x000023: 'NTT Comware',
    0x000025: 'LIXIL',
    0x00002C: 'AFT',
    0x00002E: 'SHIKOKU INSTRUMENTATION',
    0x00002F: 'AIPHONE',
    0x000034: 'MITSUBISHI ELECTRIC ENGINEERING',
    0x000035: 'Toshiba Toko Meter Systems',
    0x000036: 'NISSIN SYSTEMS',
    0x000038: 'Adosol Nissin',
    0x000039: 'Sanden',
    0x00003A: 'Sekisui House',
    0x00003B: 'Kyocera',
    0x00003C: 'Denso',
    0x00003D: 'Sumitomo Electric Industries',
    0x00003E: 'Sumitomo Electric Networks',
    0x000040: 'Hitach Hi-Tech Solutions',
    0x000041: 'Enegate',
    0x000043: 'Toshiba Digital Media Engineering',
    0x000044: 'Hitach Industrial Equipment Systems',
    0x000045: 'TIS',
    0x000047: 'NTT East',
    0x000048: 'Oki Electric Industry',
    0x00004D: 'INABA DENKI SANGYO',
    0x00004E: 'Fujitsu',
    0x00004F: 'Daiwa House Industry',
    0x000050: 'TOTO',
    0x000051: 'Fuji IT',
    0x000052: 'Osaki Electric',
    0x000053: 'Ubiquitous',
    0x000054: 'Noritz',
    0x000055: 'Familynet Japan',
    0x000056: 'iND',
    0x000057: 'Elii Power',
    0x000058: 'Mediotech',
    0x000059: 'Rinnai',
    0x00005C: 'Transboot',
    0x00005E: 'GWSOLAR',
    0x000060: 'Sony CSL',
    0x000061: 'NTT Data Intellilink',
    0x000063: 'Kawamura Electric',
    0x000064: 'OMRON',
    0x000067: 'CORONA',
    0x000068: 'Aisin Seiki',
    0x000069: 'Toshiba Lifestyle Products & Services',
    0x00006A: 'Okaya',
    0x00006B: 'ISB',
    0x00006C: 'Nichicon',
    0x00006E: 'Soundvision',
    0x00006F: 'Buffalo',
    0x000070: 'CEC',
    0x000071: 'Nihon Sangyo',
    0x000072: 'Eneres',
    0x000073: 'NEC Platforms',
    0x000075: 'RICOH IT Solutions',
    0x000076: 'TSP',
    0x000077: 'Kanagawa Institute of Technology',
    0x000078: 'Hitach Maxell',
    0x000079: 'ANRITSU Engineering',
    0x00007A: 'Zuken Elmic',
    0x00007C: 'Nihon Systemware',
    0x00007E: 'SMK',
    0x00007F: 'Anritsu Customer Support',
    0x000080: 'Tabuchi Electric',
    0x000081: 'Iwatsu Electric',
    0x000082: 'Purpose',
    0x000083: 'Melco Techno Yokohama',
    0x000084: 'Rohm',
    0x000085: 'Takaoka Toko',
    0x000086: 'NTT West',
    0x000087: 'IO Data Device',
    0x000088: 'Chofu Seisakusho',
    0x000089: 'Hokkaido Electric Industries',
    0x00008A: 'Fujitsu General',
    0x00008C: 'Kyuden Technosystems',
    0x00008D: 'NTT',
    0x00008E: 'Yamaha',
    0x00008F: 'Glamo',
    0x000090: 'Fujitsu Component',
    0x000091: 'NEC Engineering',
    0x000092: 'LSIS',
    0x000093: 'Satori Electric',
    0x000095: 'Yamato Denki',
    0x000096: 'Azbil',
    0x000097: 'Future Technology Laboratories',
    0x000099: 'Tokyo Electric Power Company Holdings',
    0x00009A: 'The Kansai Electric Power Company',
    0x00009B: 'Gastar',
    0x00009C: 'Diamond Electric Mfg',
    0x00009E: 'Yaskawa Electric',
    0x00009F: 'GS Yuasa International',
    0x0000A0: 'NTT Advance Technology',
    0x0000A1: 'Honda R&D',
    0x0000A2: 'Renaisance',
    0x0000A3: 'Chubu Electric Power Company',
    0x0000A5: 'Nichibei',
    0x0000A7: 'Jel System',
    0x0000A8: 'Smart Power System',
    0x0000A9: 'Onkyo Development & Manufacturing',
    0x0000AA: 'Ten Feet Wright',
    0x0000AC: 'IDEC',
    0x0000AD: 'Delta Electronics',
    0x0000AE: 'Shikoku Electric Power Company',
    0x0000AF: 'Takara Standard',
    0x0000B0: 'Naltec',
    0x0000B1: 'Internet Initiative Japan',
    0x0000B2: 'NF',
    0x0000B3: 'Toppers Project',
    0x0000B4: '4R Energy',
    0x0000B5: 'The Chugoku Electric Power Company',
    0x0000B6: 'Bunka Shutter',
    0x0000B7: 'Nitto Kogyo',
    0x0000B8: 'Hokkaido Electric Power Company',
    0x0000B9: 'Energy Demand Side Service Development',
    0x0000BA: 'Sankyo Tateyama',
    0x0000BB: 'Hokuriku Electric Power Company',
    0x0000BC: 'Tohoku Electric Power Company',
    0x0000BE: 'Denken',
    0x0000BF: 'Kyushyu Electric Power Company',
    0x0000C1: 'Tsuken Electric Ind',
    0x0000C2: 'Tohoku Electric Meter Industry',
    0x0000C3: 'JEMIC',
    0x0000C5: 'Sanwa Shutter',
    0x0000C9: 'Kikusui',
    0x0000CA: 'JSP',
    0x0000CB: 'Fuji Electric',
    0x0000CC: 'Hitach-Johnson Controls Air Conditioning',
    0x0000CD: 'Toclas',
    0x0000CE: 'Shindengen Electric Manufacturing',
    0x0000D0: 'Tsubamoto Chain',
    0x0000D1: 'Takebishi',
    0x0000D2: 'Chofukosan',
    0x0000D4: 'Murata Manufacturing',
    0x0000D5: 'Chosyu Industry',
    0x0000D6: 'Sassor',
    0x0000D7: 'Kaga Electronics',
    0x0000D8: 'Osaki Datatech',
    0x0000D9: 'Toshiba IT Control Systems',
    0x0000DA: 'Panasonic Commercial Equipment Systems',
    0x0000DB: 'Santech Power Japan',
    0x0000DC: 'Nihon Techno',
    0x0000DD: 'EneStone',
    0x0000DE: 'Fuji Xerox Hokkaido',
    0x0000E0: 'Looop',
    0x0000E1: 'Softbank',
    0x0000E2: 'NextDrive',
    0x0000E3: 'DDL',
    0x0000E4: 'Technoeyes',
    0x0000E5: 'Hitach Power Solutions',
    0x0000E6: 'Hokkaido Electrical Safty Services Foundation',
    0x0000E7: 'Infometis',
    0x0000E8: 'Koizumi Lighting Technology',
    0x0000E9: 'NTT Smile Energy',
    0x0000EA: 'SI Solar',
    0x0000EB: 'Nichicon Kameoka',
    0x0000EC: 'Toshiba Energy Systems & Solutions',
    0x0000ED: 'Infini',
    0x0000EE: 'Tessera Technology',
    0x0000EF: 'Toyoda industories',
    0x0000F0: 'Kaneka',
    0x0000F1: 'Laplace Systems',
    0x0000F2: 'Energy Solutions',
    0x0000F3: 'Energy Gateway',
    0x0000F4: 'Denso Aircool',
    0x0000F6: 'Field Logic',
    0x0000F7: 'JCity',
    0x0000F8: 'Avenir',
    0x0000F9: 'Toho Electronics',
    0x0000FA: 'Plat Home',
    0x0000FB: 'CICO',
    0x0000FC: 'Fuji Industrial',
    0xFFFFFE: 'Undefined',
    0xFFFFFF: 'Experimental'
}

ENL_ON = 0x30
ENL_OFF = 0x31
ENL_STATUS = 0x80
ENL_UID = 0x83
ENL_CUMULATIVE_POWER = 0x85
ENL_MANUFACTURER = 0x8A
ENL_CUMULATIVE_RUNTIME = 0x9A
ENL_SETMAP = 0x9E
ENL_GETMAP = 0x9F



from pychonet.EchonetInstance import EchonetInstance

MEASURED_TEMP = 0xE0

# ----- Tempereature Sensor -------
def _0011E0(edt):
        return float(int.from_bytes(edt, 'big')) / 10

class TemperatureSensor(EchonetInstance):
    EPC_FUNCTIONS = {
        0xE0: _0011E0,
    }

    def __init__(self, host, api_connector = None, instance = 0x1):
        self.eojgc = 0x00
        self.eojcc = 0x11
        EchonetInstance.__init__(self, host, self._eojgc, self._eojcc, instance, api_connector)

    def getMeasuredTemperature(self):
        return self.getMessage(MEASURED_TEMP)



from datetime import datetime, timezone
from pychonet.lib.const import MANUFACTURERS

#------------ EPC GENERIC FUNCTIONS -------
def _int(edt):
    return int.from_bytes(edt, 'big')

# Check status of Echonnet Instance
#----------------- EPC SUPER FUNCTIONS -----------------------------
def _0080(edt):
    ops_value = int.from_bytes(edt, 'big')
    return ('On' if ops_value == 0x30 else 'Off')

def _009X(edt):
    payload = []
    if len(edt) < 17:
        for i in range (1, len(edt)):
            payload.append(edt[i])
        return payload

    for i in range (1, len(edt)):
        code = i-1
        binary = '{0:08b}'.format(edt[i])[::-1]
        for j in range (0, 8):
            if binary[j] == "1":
                EPC = (j+8) * 0x10 + code
                payload.append(EPC)
    return payload

def _0083(edt):
    if edt is not None:
        if len(edt) > 1:
            ops_value = edt[1:].hex()
        else:
            ops_value = None
        return ops_value
    return None

def _008A(edt): #manufacturer
    id = int.from_bytes(edt, 'big')
    if id in MANUFACTURERS.keys():
        return MANUFACTURERS[id]
    return id

def _009A(edt): #cumulative runtime
    if len(edt) > 1:
        value = int.from_bytes(edt[1:], 'big')
        time_period_multiplier = 0
        if edt[0] == 0x41:
            time_period_multiplier = 1 #measurement is in seconds
        elif edt[0] == 0x42:
            time_period_multiplier = 60 #measurement is in minutes
        elif edt[0] == 0x43:
            time_period_multiplier = 3600 #measurement is in hours
        elif edt[0] == 0x44:
            time_period_multiplier = 3600*24 #measurement is in days
        return value * time_period_multiplier
    return None

EPC_SUPER_FUNCTIONS = {
    0x80: _0080,
    0x83: _0083,
    0x85: _int,
    0x8A: _008A,
    0x9A: _009A,
    0x9E: _009X,
    0x9F: _009X
}


# ------- EPC FUNCTIONS -------------------------------------------------
# -----------------------------------------------------------------------

def _0260EO(edt):
    op_mode = int.from_bytes(edt, 'big')
    values = {
       0x41: 'open',
       0x42: 'close',
       0x43: 'stop'
    }
    return values.get(op_mode, "invalid_setting")

# ----- Electric Lock Class -------
def _026FEX(edt):
    op_mode = int.from_bytes(edt, 'big')
    values = {
       0x41: 'lock',
       0x42: 'unlock'
    }
    return values.get(op_mode, "invalid_setting")

def _026FE3(edt):
    op_mode = int.from_bytes(edt, 'big')
    values = {
       0x41: 'open',
       0x42: 'closed'
    }
    return values.get(op_mode, "invalid_setting")

def _026FE4(edt):
    op_mode = int.from_bytes(edt, 'big')
    values = {
       0x41: 'occupant',
       0x42: 'non-occupant'
    }
    return values.get(op_mode, "invalid_setting")

def _026FE5(edt):
    op_mode = int.from_bytes(edt, 'big')
    values = {
       0x40: 'normal',
       0x41: 'break_open',
       0x42: 'door_open',
       0x43: 'maunal_unlocked',
       0x44: 'tampered'
    }
    return values.get(op_mode, "invalid_setting")

def _026FE6(edt):
    op_mode = int.from_bytes(edt, 'big')
    values = {
       0x41: 'on',
       0x42: 'off'
    }
    return values.get(op_mode, "invalid_setting")

def _026FE7(edt):
    op_mode = int.from_bytes(edt, 'big')
    values = {
       0x41: 'normal',
       0x42: 'battery_replacement_notification'
    }
    return values.get(op_mode, "invalid_setting")

  #--- Low voltage smart meter class

def _0288E1(edt):
    op_mode = int.from_bytes(edt, 'big')
    values = {
        0x00: 1,
        0x01: 0.1,
        0x02: 0.01,
        0x03: 0.001,
        0x04: 0.0001,
        0x0A: 10,
        0x0B: 100,
        0x0C: 1000,
        0x0D: 10000,
    }
    return values.get(op_mode, "invalid_setting")


# ----- Low voltage smart electric energy meter -------
def _0288E7(edt):
    value = int.from_bytes(edt, 'big', signed=True)
    return value


def _0288E8(edt):
    r_phase = float(int.from_bytes(edt[0:2], 'big', signed=True)) / 10 #R Phase
    t_phase = float(int.from_bytes(edt[2:4], 'big', signed=True)) / 10 #T Phase
    return {"r_phase_amps": r_phase, "t_phase_amps": t_phase}

def _0288EA(edt):
    print(edt)
    year = int.from_bytes(edt[0:2], 'big')
    month = int.from_bytes(edt[2:3], 'big')
    day = int.from_bytes(edt[3:4], 'big')
    hour = int.from_bytes(edt[4:5], 'big')
    minute = int.from_bytes(edt[5:6], 'big')
    second = int.from_bytes(edt[6:7], 'big')
    culmative = int.from_bytes(edt[7:], 'big')
    time = datetime(year, month, day, hour,minute,second)
    return {'time': datetime(year, month, day, hour,minute,second).isoformat(), 'culmative_value': culmative}

# ----- General lighting class -------
def _0290B1(edt):
    op_mode = int.from_bytes(edt, 'big')
    values = {
       0x40: 'other',
       0x41: 'incandescent_lamp_color',
       0x42: 'white',
       0x43: 'daylight_white',
       0x43: 'daylight_color',
    }
    return values.get(op_mode, "invalid_setting")

# EPC_FUNCTIONS = {
#     0x00: {
#         0x11:{
#             0xE0: _0011E0,
#             0XE1: _int
#         }
#     },
#     0x01: {
#         0x30:{
#         	0xA0: _0130A0,
#             0xA1: _0130A1,
#             0xA3: _0130A3,
#             0xA4: _0130A4,
#             0xA5: _0130A5,
#             0xAA: _0130AA,
#             0xB0: _0130B0,
#             0xB3: _int,
#             0xBB: _int,
#             0xBE: _int
#         }
#     },
#     0x02: {
#         0x60: {
#             0xE0: _0260EO
#         },
#         0x6F: {
#             0xE0: _026FEX,
#             0xE1: _026FEX,
#             0xE2: _026FEX,
#             0xE3: _026FE3,
#             0xE4: _026FE4,
#             0xE5: _026FE5,
#             0xE6: _026FE6,
#             0xE7: _026FE7
#         },
#         0x79: {
#             0xE0: _int,
#             0xE1: _int
#         },
#         0x7d: {
#             0xE4: _int,
#             0xCF: _int
#         },
#         0x88: {
#             0xd7: _int, #number of effective digits in culmative amout of energy
#             0xe0: _int, #measured culmative amount of electricity
#             0xe1: _0288E1, #multiplying factor
#             # 0xe2:
#             0xe5: _int,
#             0xe7: _0288E7, #measured instantaneous electric energy
#             0xe8: _0288E8, #measured instantanenous currents
#             0xea: _0288EA
#         },
#         0x90: {
#             0xB1: _0290B1
#         }
#     }
# }



from pychonet.echonetapiclient import ECHONETAPIClient
from .EchonetInstance import EchonetInstance
from .HomeAirConditioner import HomeAirConditioner
from .HomeSolarPower import HomeSolarPower
from .ElectricVehicleCharger import ElectricVehicleCharger
from .StorageBattery import StorageBattery
from .TemperatureSensor import TemperatureSensor
from pychonet.lib.eojx import EOJX_CLASS

def Factory(host, server, eojgc, eojcc, eojci= 0x01):

    instance = EOJX_CLASS[eojgc][eojcc]

    """Factory Method"""
    instances = {
        'Home air conditioner': HomeAirConditioner,
        'Home solar power generation': HomeSolarPower,
        'Electric vehicle charger/discharger': ElectricVehicleCharger,
        'Temperature sensor': TemperatureSensor,
        'Storage Battery': StorageBattery
    }
    instance_object = instances.get(instance, None)
    if instance_object is not None:
        return instance_object(host, server, eojci)

    return EchonetInstance(host, server, eojgc, eojcc, eojci)



from pychonet.EchonetInstance import EchonetInstance

class HomeSolarPower(EchonetInstance):
    def __init__(self, host, api_connector, instance = 0x1):
        self.eojgc = 0x02
        self.eojcc = 0x79
        EchonetInstance.__init__(self, host, self._eojgc, self._eojcc, instance, api_connector)

    def getMeasuredInstantPower(self):
        return int.from_bytes(self.getMessage(0xE0), 'big')

    def getMeasuredCumulPower(self):
        return int.from_bytes(self.getMessage(0xE1), 'big')



from pychonet.EchonetInstance import EchonetInstance

"""Class for Electric Vehicle Charger Objects"""
class ElectricVehicleCharger(EchonetInstance):

    def __init__(self, host, api_connector = None, instance = 0x1):
        self.eojgc = 0x02 # Housing/facility-related device group
        self.eojcc = 0x7e # Electric vehicle charger/discharger’
        EchonetInstance.__init__(self, host, self._eojgc, self._eojcc, instance, api_connector)
